@charset "UTF-8";
@namespace d url(http://www.apple.com/DTDs/DictionaryService-1.0.rng);

d|entry {
}

h1	{
	font-size: 150%;
}

h3	{
	font-size: 100%;
}

.ex, .m, .m0, .m1, .m2, .m3, .m4, .m5, .m6, .m7, .m8, .m9 {
	display: block;
}

.m  { margin-left: 0em; }
.m0 { margin-left: 0em; }
.m1 { margin-left: 1em; }
.m2 { margin-left: 2em; }
.m3 { margin-left: 3em; }
.m4 { margin-left: 4em; }
.m5 { margin-left: 5em; }
.m6 { margin-left: 6em; }
.m7 { margin-left: 7em; }
.m8 { margin-left: 8em; }
.m9 { margin-left: 9em; }

.ex + br,
.k + br { display: none; }

.c {
	color: green;
}

.p {
	font-style: italic;
	color: green;
}

.ex {
	color: #666;
}

.u {
	text-decoration: underline;
}

/* xdxf support */
.k {
	color: black;
	font-weight: bold;
	display: block;
}
.tr {
	color: black;
}
.abr {
	color: #008000;
	font-style: italic;
}
.hideextra .extra {
	display: none;
}
.stress {
	color: #FF0000;
}
.kref {
	color: #000080;
	text-decoration: none;
}
.pr {
	color: #000080;
	white-space: nowrap;
	text-decoration: none;
	overflow: hidden;
	text-overflow: ellipsis;
	padding-right: 1ex;
}
